/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.container.lifecycle;

import java.io.File;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.management.DeploymentException;
import javax.jbi.management.LifeCycleMBean;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.util.PackageHelper;

/**
 * This class manage the life cycle of deployed service units
 * 
 * @author Olivier Fabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class ServiceUnitLifeCycle implements LifeCycleMBean {

    private AdminService adminService;

    private ServiceUnit serviceUnit;

    private String serviceUnitRootPath;

    private Jbi jbiDescriptor;

    private ServiceUnitManager serviceUnitManager;

    private String name;

    protected String state;

    public ServiceUnitLifeCycle(ServiceUnit serviceUnit, String serviceUnitRootPath,
            AdminService adminService) {
        super();
        this.adminService = adminService;
        this.serviceUnit = serviceUnit;
        this.serviceUnitRootPath = serviceUnitRootPath;
        if (serviceUnit != null && serviceUnit.getIdentification() != null) {
            this.name = serviceUnit.getIdentification().getName();
        }
        state = SHUTDOWN;
    }

    /**
     * Returns the ServiceUnitManager associated with the service unit's target
     * component
     * 
     * @return null if there's no Service Unit Manager
     */
    public ServiceUnitManager getServiceUnitManager() {
        if (this.serviceUnitManager == null) {
            Installer installer = adminService.getInstallerByName(serviceUnit.getTarget()
                    .getComponentName());

            if (installer != null) {
                this.serviceUnitManager = installer.getComponent().getServiceUnitManager();
            }
        }

        return this.serviceUnitManager;
    }

    /**
     * Return the name of the targeted component.
     * 
     * @return the name of the targeted component
     */
    public String getTargetComponentName() {
        return serviceUnit.getTarget().getComponentName();
    }

    public String getServiceUnitRootPath() {
        return serviceUnitRootPath;
    }

    public String getSuName() {
        return name;
    }

    public synchronized Jbi getServiceUnitDescriptor() {
        if (this.jbiDescriptor == null) {
            try {
                this.jbiDescriptor = PackageHelper
                        .loadDescriptor(new File(this.serviceUnitRootPath));
            } catch (Exception e) {
                // not possible, the descriptor has been already loaded by the
                // target component
            }
        }
        return jbiDescriptor;
    }

    public void init() throws DeploymentException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().init(name, serviceUnitRootPath);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = STOPPED;
    }

    public String getCurrentState() {
        return state;
    }

    public void shutDown() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().shutDown(name);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = SHUTDOWN;
    }

    public void start() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().start(name);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = STARTED;
    }

    public void stop() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().stop(name);
        } catch (DeploymentException e) {
            state = UNKNOWN;
            throw e;
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
        state = STOPPED;
    }

}
