/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: InstallDirectoryScanner.java,v 1.2 2006/04/07 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.autoload;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.TimerTask;

import org.ow2.petals.util.NioUtil;

/**
 * Periodically check if new Install or Deploy elements are presents in the
 * autoload directories.
 * 
 * @author alouis
 * 
 */
public class InstallDirectoryScanner extends TimerTask {

    private final File installDir;

    private final AutoLoaderServiceImpl autoLoader;

    private final ArchiveFileFilter archiveFilter;

    private final List<File> previousArchives = new ArrayList<File>();

    public InstallDirectoryScanner(AutoLoaderServiceImpl autoLoader, File installDir) {
        this.installDir = installDir;
        this.autoLoader = autoLoader;
        this.archiveFilter = ArchiveFileFilter.getInstance();
    }

    @Override
    @edu.umd.cs.findbugs.annotations.SuppressWarnings(value = { "RCN_REDUNDANT_NULLCHECK_WOULD_HAVE_BEEN_A_NPE" }, justification = "It's needed to check 'newInstall' to prevent NPE. It can be null if 'this.installDir' is not a directory.")
    public void run() {
        final File[] newInstalls = this.installDir.listFiles(this.archiveFilter);
        final List<File> newInstallList = new ArrayList<File>();

        synchronized (this.previousArchives) {

            // get only new archives
            for (final File archive : newInstalls) {
                if (!this.previousArchives.contains(archive)) {
                    newInstallList.add(archive);
                }
            }
            this.previousArchives.clear();

            // remove archives that are not completely copied in the install
            // directory
            final List<File> deleteInstallList = new ArrayList<File>();
            for (final File newArchive : newInstallList) {
                if (!NioUtil.checkAccess(newArchive)) {
                    deleteInstallList.add(newArchive);
                }
            }
            newInstallList.removeAll(deleteInstallList);

            if (!newInstallList.isEmpty()) {
                this.autoLoader.install(newInstallList);
                this.previousArchives.addAll(newInstallList);
            }
        }
    }
}
