/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task download entity (component, shared lib, service assembly) archive
 * into the work directory, only if the URL protocol is http: or ftp: <br/>
 * Inputs : archive {@link URI} <br/>
 * Outputs : downloaded archive {@link URI}<br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class DownloadTask extends AbstractFileManipulationTask {

    public DownloadTask(LoggingUtil log, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
    }

    public void execute(Context context) throws Exception {
        URL archiveUrl = context.getArchiveUrl();
        URL workArchiveUrl = this.downloadEntityPackage(archiveUrl);
        context.setArchiveUrl(workArchiveUrl);
    }

    private URL downloadEntityPackage(URL archiveUrl) throws IOException {
        InputStream inputStream = null;
        FileOutputStream fileOutputStream = null;
        URL url = null;
        try {
            final String path = archiveUrl.getPath();
            final String fileName = path.substring(path.lastIndexOf('/') + 1);
            inputStream = archiveUrl.openStream();
            final File outputFile = new File(this.getWorkDirectory(), fileName);
            fileOutputStream = new FileOutputStream(outputFile);
            IOUtils.copy(inputStream, fileOutputStream);
            url = outputFile.toURI().toURL();
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
                if (fileOutputStream != null) {
                    fileOutputStream.close();
                }
            } catch (IOException e) {
                this.log.warning("Failed to clean the downloaded archive '"
                        + archiveUrl.toExternalForm() + "'", e);
            }
        }
        return url;
    }

    public void undo(Context context) throws Exception {
        URL downloadUrl = context.getArchiveUrl();
        if (downloadUrl != null) {
            try {
                FileUtils.forceDelete(new File(downloadUrl.getPath()));
            } catch (IOException e) {
                this.log.error("Failed to clean downloaded artefact " + downloadUrl);
            }
        }
    }

}
