/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: UnregisterSALifeCycleTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.undeploy;

import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task removes the service assembly life cycle from the manager service SA
 * Map<br/>
 * Inputs : entity name {@link String}<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class UnregisterSALifeCycleTask extends AbstractLoggableTask {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    /**
     * JBI Container :: Fractal container manager
     */
    protected ContainerService containerService;

    public UnregisterSALifeCycleTask(LoggingUtil log, AdminService adminService,
            ContainerService containerService) {
        super(log);
        this.adminService = adminService;
        this.containerService = containerService;
    }

    public void execute(Context context) {
        String name = context.getEntityName();
        if (name != null) {
            try {
                this.adminService.unregisterServiceAssembly(name);
            } catch (ManagementException e) {
                this.log.error("Unable to unregister a service assembly", e);
            }
        }
        /*
         * Remove the fractal component
         */
        ServiceAssemblyLifeCycle saLifeCycle = context.getSaLifeCycle();
        if (saLifeCycle != null) {
            try {
                this.containerService.removeServiceAssemblyLifeCycle(saLifeCycle);
            } catch (PetalsException e) {
                this.log.error("Unable to remove a fractal service assembly lifecycle", e);
            }
        }
    }

    public void undo(Context context) throws Exception {
        // TODO : register again the service assembly lifecycle
    }

}
