/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.installation;

import java.io.File;
import java.util.List;

import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.StringHelper;

/**
 * 
 * @author ofabre
 * 
 */
public class InstallationUtils {

    /**
     * Check the classpath element of the jbi descriptor
     * 
     * @param filePackage
     *            the entity package root
     * @param classPath
     *            a List of classpath elements to check
     * @throws PetalsException
     *             if one of the classpath elements is null, empty or or missing
     */
    public static final void checkClassPathElements(File filePackage, List<String> classPath)
            throws PetalsException {

        for (String pathElement : classPath) {
            // empty class-path elements is not alloweds
            if (StringHelper.isNullOrEmpty(pathElement)) {
                throw new PetalsException("Found empty class-path element in the JBI descriptor");
            }

            // check if the specified path element exists within archive
            if (!".".equals(pathElement) && !hasEntry(filePackage, pathElement)) {
                throw new PetalsException("The class-path element '" + pathElement
                        + "' declared in the " + "JBI descriptor cannot be resolved");
            }
        }
    }

    private static final boolean hasEntry(File filePackage, String pathElement) {
        File artifactFile = new File(filePackage, pathElement);
        return artifactFile.exists();
    }
}
