/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.component;

import javax.management.ObjectName;

import org.ow2.petals.container.ContainerService;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task unregister component installer from JMX server.<br/>
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * Inputs : component name <br/>
 * Outputs : component installer {@link ObjectName}
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class UnregisterInstallerTask extends AbstractLoggableTask {

    private final AdminService adminService;

    private final ContainerService containerService;

    public UnregisterInstallerTask(LoggingUtil log, AdminService adminService,
            ContainerService containerService) {
        super(log);
        this.adminService = adminService;
        this.containerService = containerService;
    }

    public void execute(Context context) throws Exception {
        if (context.getEntityName() != null) {
            String componentName = context.getEntityName();
            if (componentName != null) {
                // Unregister the Installer MBean
                try {
                    this.adminService.unregisterInstaller(componentName);
                } catch (ManagementException e) {
                    this.log.error("Installer MBean can not be unregistered", e);
                }
                // Remove the Fractal installer
                try {
                    this.containerService.removeInstaller(componentName);
                } catch (PetalsException e) {
                    this.log.error("Fractal installer can not be removed", e);
                }
            }

        }
    }

    public void undo(Context context) throws Exception {
        // TODO : recreate and register the installer
    }

}
