/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.sl;

import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.system.systemstate.SystemStateService;
import org.ow2.petals.systemstate.SystemStateException;
import org.ow2.petals.systemstate.generated.SharedLibrary;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task removes the shared lib from the state holder file <br/>
 * Inputs : shared lib name <br/>
 * * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * Outputs : {@link SharedLibraryState}
 * 
 * @author ofabre - EBM Websourcing
 * @author fgardes - EBM Websourcing
 */
public class RemoveSLStateHolderTask extends AbstractLoggableTask {

    private final SystemStateService recoveryService;

    public RemoveSLStateHolderTask(LoggingUtil log, SystemStateService recoveryService) {
        super(log);
        this.recoveryService = recoveryService;
    }

    public void execute(Context context) {
        String slName = context.getEntityName();
        String version = context.getEntityVersion();
        SharedLibrary sharedLibraryStateHolder = null;

        if (slName != null) {
            try {
                sharedLibraryStateHolder = this.recoveryService.deleteSharedLibraryStateHolder(
                        slName, version);
                context.setSharedLibraryStateHolder(sharedLibraryStateHolder);
            } catch (SystemStateException e) {
                this.log.error("The shared library '" + slName + "' with version '" + version
                        + "' state holder can not be removed", e);
            }
        }
    }

    public void undo(Context context) throws Exception {
        // TODO : recreate the state holder
    }

}
