/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.system.repository.artifact;

import java.io.File;
import java.util.List;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.util.LoggingUtil;

/**
 * The artifact repository service.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = ArtifactRepositoryService.class))
public class FractalArtifactRepositoryService implements ArtifactRepositoryService {

    private ArtifactRepositoryService delegate;

    @Requires(name = "configuration", signature = ConfigurationService.class)
    private ConfigurationService configurationService;

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    public FractalArtifactRepositoryService() {
    }

    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        this.log = new LoggingUtil(logger);
        this.log.start();
        
        File rootPath = new File(this.configurationService.getContainerConfiguration()
                .getRootDirectoryPath(), "artifacts");
        this.delegate = new LocalArtifactRepositoryService(rootPath);
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        this.log.end();
        this.delegate = null;
    }

    public List<Artifact> getComponents() {
        return delegate.getComponents();
    }

    public File getComponentsPath() {
        return delegate.getComponentsPath();
    }

    public File getRootPath() {
        return delegate.getRootPath();
    }

    public boolean addArtifact(Artifact artifact) throws PetalsException {
        return delegate.addArtifact(artifact);
    }

    public List<String> getComponentsName() {
        return delegate.getComponentsName();
    }

    public List<String> getServiceAssembliesName() {
        return delegate.getServiceAssembliesName();
    }

    public File getServiceAssemblyPath() {
        return delegate.getServiceAssemblyPath();
    }

    public List<String> getSharedLibrariesName() {
        return delegate.getSharedLibrariesName();
    }

    public File getSharedLibraryPath() {
        return delegate.getSharedLibraryPath();
    }

    public List<Artifact> getServiceAssemblies() {
        return delegate.getServiceAssemblies();
    }

    public List<Artifact> getSharedLibraries() {
        return delegate.getSharedLibraries();
    }

}
